#coding = utf-8
# from ..base import Dat
import time
import array

class CBNGramFeature:

    SENTENCE_BOUNDARY = '#'
    SEPERATOR = "_"
    maxLength = 0
    uniBases = []
    biBases = []
    datSize = []
    dat = []
    values = {}

    def __init__(self, myDat, model):
        self.SEPERATOR = ' '
        self.datSize = myDat.getDatSize()
        self.dat = myDat.getDat()
        self.model = model
        self.maxLength = 10000
        self.uniBases = [0 for i in range(self.maxLength + 2)]
        self.biBases = [0 for i in range(self.maxLength + 4)]
    
    def addValues(self, valueOffset, base, dele, tmp):
        ind = self.dat[2 * base] + dele
        if(ind >= self.datSize or self.dat[2 * ind + 1] != base):
            return
        # offset = self.dat[2 * ind]
        weightOffset = self.model.l_size * self.dat[2 * ind]
        if(self.model.l_size == 4):
            # pass
            tmp[0] += self.model.fl_weights[weightOffset]
            tmp[1] += self.model.fl_weights[weightOffset + 1]
            tmp[2] += self.model.fl_weights[weightOffset + 2]
            tmp[3] += self.model.fl_weights[weightOffset + 3]
            # self.values[valueOffset] += self.model.fl_weights[weightOffset]
            # self.values[valueOffset + 1] += self.model.fl_weights[weightOffset + 1]
            # self.values[valueOffset + 2] += self.model.fl_weights[weightOffset + 2]
            # self.values[valueOffset + 3] += self.model.fl_weights[weightOffset + 3]
        else:
            for i in range(self.model.l_size):
                self.values[valueOffset + i] += self.model.fl_weights[weightOffset + i]
    
    def findBases(self, datSize, ch1, ch2):
        result = []
        uniBase = 0
        biBase = 0
        ch1 = ord(ch1)
        ch2 = ord(ch2)
        if(ch1 > 32 and ch1 < 128):
            ch1 += 65248
        if(ch2 > 32 and ch2 < 128):
            ch2 += 65248
        if(ch1 >= datSize or 2 *self.dat[2 * ch1 + 1] != 0):
            # uniBase = -1
            # biBase = -1
            # result = (-1, -1)
            return -1, -1

        uniBase = self.dat[2 * ch1] + ord(self.SEPERATOR)

        ind = self.dat[2 * ch1] + ch2
        if(ind >= datSize or self.dat[2 * ind + 1] != ch1):
            # biBase = -1
            # result = (uniBase, -1)
            return uniBase, -1

        biBase = self.dat[2 * ind] + ord(self.SEPERATOR)
        # result = (uniBase, biBase)
        return uniBase, biBase

    def putValues(self, sequence, mylen, values):
        if(mylen >= self.maxLength):
            print("larger than max")
            return 1

        # start = time.clock()
        self.values = values
        self.uniBases[0], self.biBases[0] = self.findBases(self.datSize, self.SENTENCE_BOUNDARY, self.SENTENCE_BOUNDARY)
        # self.uniBases[0] = result[0]
        # self.biBases[0] = result[1]
        
        self.uniBases[0], self.biBases[1] = self.findBases(self.datSize, self.SENTENCE_BOUNDARY, sequence[0])
        # self.uniBases[0] = result[0]
        # self.biBases[1] = result[1]

        for i in range(mylen - 1):
            self.uniBases[i + 1], self.biBases[i + 2] = self.findBases(self.datSize, sequence[i], sequence[i+1])
            # self.uniBases[i+1] = result[0]
            # self.biBases[i+2] = result[1]

        self.uniBases[mylen], self.biBases[mylen + 1] = self.findBases(self.datSize, sequence[mylen - 1], self.SENTENCE_BOUNDARY)
        # self.uniBases[mylen] = result[0]
        # self.biBases[mylen+1] = result[1]
        
        self.uniBases[mylen+1], self.biBases[mylen+2] = self.findBases(self.datSize, self.SENTENCE_BOUNDARY, self.SENTENCE_BOUNDARY)
        # self.uniBases[mylen+1] = result[0]
        # self.biBases[mylen+2] = result[1]
        
        # end = time.clock()
        # print "FB Time used: %f s" % (end - start)
        # start = time.clock()
        for i in range(mylen):
            tmp = [0, 0, 0, 0]
            valueOffset = i * self.model.l_size
            if((self.uniBases[i + 1]) != -1):
                self.addValues(valueOffset, self.uniBases[i + 1], 49, tmp)
            if((self.uniBases[i]) != -1):
                self.addValues(valueOffset, self.uniBases[i], 50, tmp)
            if((self.uniBases[i + 2]) != -1):
                self.addValues(valueOffset, self.uniBases[i + 2], 51, tmp)
            if(self.biBases[i + 1] != -1):
                self.addValues(valueOffset, self.biBases[i + 1], 49, tmp)
            if((self.biBases[i + 2]) != -1):
                self.addValues(valueOffset, self.biBases[i + 2], 50, tmp)
            if((self.biBases[i]) != -1):
                self.addValues(valueOffset, self.biBases[i], 51, tmp)
            if((self.biBases[i + 3]) != -1):
                self.addValues(valueOffset, self.biBases[i + 3], 52, tmp)
            self.values[valueOffset] += tmp[0]
            self.values[valueOffset + 1] += tmp[1]
            self.values[valueOffset + 2] += tmp[2]
            self.values[valueOffset + 3] += tmp[3]

        # end = time.clock()
        # print "AV Time used: %f s" % (end - start)
        return 0



from .base.Dat import Dat


class Postprocesser():

    def __init__(self, filename, tag, isTxt):
        self.tag = tag
        self.p_dat = Dat(filename)


    def adjustSeg(self, sentence):
        if(self.p_dat is None):
            return
        i = 0
        while(i < len(sentence)):
            tmp = sentence[i]
            if(self.p_dat.getInfo(tmp) >= 0):
                i += 1
                continue
            tmpVec = []
            for j in range(i + 1, len(sentence)):
                tmp += sentence[j]
                if(self.p_dat.getInfo(tmp) >= 0):
                    break
                tmpVec.append(tmp)
            vecSize = len(tmpVec)
            
            for k in range(vecSize-1, -1, -1):
                tmp = tmpVec[k]
                if(self.p_dat.match(tmp) != -1):
                    sentence[i] = tmp
                    del sentence[i+1:i+k+2]
                    break
            i += 1


    def adjustTag(self, sentence):
        if(self.p_dat is None):
            return
        i = 0
        while(i < len(sentence)):
            tmp = sentence[i][0]
            if(self.p_dat.getInfo(tmp) >= 0):
                i+=1
                continue

            tmpVec = []
            for j in range(i+1, len(sentence)):
                tmp += sentence[j][0]
                if(self.p_dat.getInfo(tmp) >= 0):
                    break
                tmpVec.append(tmp)
            vecSize = len(tmpVec)

            for k in range(vecSize-1, -1, -1):
                tmp = tmpVec[k]
                if(self.p_dat.match(tmp) != -1):
                    sentence[i] = (tmp, '_', self.tag)
                    del sentence[i+1:i+k+2]
                    # sentence[i][2] = self.tag;
                    break
            i+=1



import time

class AlphaBeta:
    value = 0
    nodeId = 0
    labelId = 0

    def __init__(self):
        self.value = 0
        self.nodeId = -2
        self.labelId = 0

    def dbDecode(self, l_size, llWeights, nodeCount, nodes, values, alphas, result, preLabels, allowedLabelLists):
        nodeId = 0
        pNodeId = []
        pPreLabel = []
        pAllowedLabel = []
        k = 0
        j = 0
        tmp = AlphaBeta()
        best = (0, -1, 0)
        preAlpha = AlphaBeta()
        
        score = 0 
        index = 0
        index2 = 0
        index3 = 0

        # print nodeCount
        
        for i in range(nodeCount):
            alphas.append({})
            # start = time.clock()
            pAllowedLabel = allowedLabelLists[i]
            for j in pAllowedLabel:
                if(j == -1):
                    break
                tmp = (0, -2, 0)
                nodeId = nodes[i].predecessors
                pPreLabel = preLabels[j]
                if(nodeId >= 0):
                    for k in pPreLabel:
                        if(k == -1):
                            break
                        if(k not in alphas[nodeId]):
                            continue
                        preAlpha = alphas[nodeId][k]
                        # print nodeId, k, preAlpha.nodeId
                        if(preAlpha[1] == -2):
                            continue
                        score = preAlpha[0] + llWeights[k*l_size + j]
                        # print j, k, score
                        if((tmp[1] < 0) or (score > tmp[0])):
                            tmp = (score, nodeId, k)
                # tmp = (tmp[0] + values[i*l_size + j], -1, tmp[2])
                if((nodes[i].type == 1) or (nodes[i].type == 3)):
                    tmp = (tmp[0] + values[i*l_size + j], -1, tmp[2])
                else:
                    tmp = (tmp[0] + values[i*l_size + j], tmp[1], tmp[2])
                if(nodes[i].type >= 2):
                    if((best[1] == -1) or best[0] < tmp[0]):
                        best = (tmp[0], i, j)
                # print "%d %d %d %d %d" % (i, j, tmp.value, tmp.nodeId, tmp.labelId)
                alphas[i][j] = tmp
        tmp = best
        while(tmp[1] >= 0):
            result[tmp[1]] = tmp[2]
            if(tmp[2] in alphas[tmp[1]]):
                tmp = alphas[tmp[1]][tmp[2]]
            else:
                break
        return best[0]



import struct
import binascii
import codecs

class CBModel:
    DEC = 1000
    l_size = 0
    f_size = 0
    ll_weights = []
    fl_weights = []
    ave_ll_weights = []
    ave_fl_weights = []

    # def __init__(self, l, f):
    #     self.l_size = l
    #     self.f_size = f
    #     self.ll_weights = [0 for i in range(l * l)]
    #     self.fl_weights = [0 for i in range(f * l)]
    #     self.ave_ll_weights = [0.0 for i in range(l * l)]
    #     self.ave_fl_weights = [0.0 for i in range(f * l)]

    def reset_ave_weights(self):
        self.ave_ll_weights = [0.0 for i in range(l * l)]
        self.ave_fl_weights = [0.0 for i in range(f * l)]
    
    def update_ll_weights(self, i, j, delta, steps):
        ind = i * self.l_size + j
        self.ll_weights[ind] += delta
        self.ave_ll_weights[ind] += steps * delta

    def update_fl_weights(self, i, j, delta, steps):
        ind = i * self.l_size + j
        self.fl_weights[ind] += delta
        self.ave_fl_weights[ind] += steps * delta
    
    def average(self, step):
        for i in range(self.l_size * self.f_size):
            self.fl_weights[i] = int((float(self.fl_weights[i]) - self.ave_fl_weights[i] / float(step)) * self.DEC + 0.5)

        for i in range(self.l_size * self.l_size):
            self.ll_weights[i] = int((float(self.ll_weights[i]) - self.ave_ll_weights[i] / float(step)) * self.DEC + 0.5)
    
    def byteToInt(self, s):
        ans = ""+s[6]+s[7]+s[4]+s[5]+s[2]+s[3]+s[0]+s[1]
        ans = int(ans, 16)
        # print ans
        # ans = 0        
        # for i in range(7, -1, -2):
        #     ans = int(ans * 256) + int(s[i-1], 16) * 16 + int(s[i], 16)
        # print ans
        if(s[6] >= '8'):
            return -((1 << 32) - ans)
        return ans
    
    def __init__(self, filename):
        inputfile = open(filename, "rb")
        temp = inputfile.read(4)
        temp = codecs.decode(binascii.hexlify(temp))
        self.l_size = self.byteToInt(temp)
        temp = inputfile.read(4)
        temp = codecs.decode(binascii.hexlify(temp))
        self.f_size = self.byteToInt(temp)
        
        # self.ll_weights = [0 for i in range(self.l_size * self.l_size)]
        temp = inputfile.read(4 * self.l_size * self.l_size)
        # for i in range(len(self.ll_weights)):
        self.ll_weights = struct.unpack("<"+str(self.l_size * self.l_size)+"i", temp)
        self.ll_weights = tuple(self.ll_weights)

        # self.fl_weights = [0 for i in range(self.f_size * self.l_size)]
        temp = inputfile.read(4 * self.f_size * self.l_size)

        # for i in range(len(self.fl_weights)):
        self.fl_weights = struct.unpack("<"+str(self.f_size * self.l_size)+"i", temp)
        self.fl_weights = tuple(self.fl_weights)
        # print len(self.fl_weights)

        inputfile.close()

if __name__ == '__main__':
    mod = CBModel("cws_model.bin")
    print(len(mod.fl_weights))




#coding = utf-8
import struct
import os


class Dat:

    def __init__(self, filename):
        inputfile = open(filename, "rb")
        self.datSize = int(os.path.getsize(filename) / 8)
        s = inputfile.read(8 * self.datSize)
        tmp = "<"+str(self.datSize*2)+"i"
        self.dat = struct.unpack(tmp, s)
        self.dat = tuple(self.dat)
        inputfile.close()

    
    def printDat(self, filename):
        f = open(filename, "w")
        for i in range(self.datSize):
            f.write(""+self.dat[2 * i]+" "+self.dat[2 * i + 1]+"\n")
        f.close()
    
    def getIndex(self, base, character):
        ind = self.dat[2 * base] + character
        if((ind >= self.datSize) or self.dat[2 * ind + 1] != base):
            return -1
        return ind
    
    def search(self, sentence, bs, es):
        bs = []
        es = []
        empty = True
        for offset in range(len(sentence)):
            preBase = 0
            preInd = 0
            ind = 0
            for i in range(offset, len(sentence)):
                ind = preBase + sentence[i]
                if(ind < 0 or ind >= self.datSize or self.dat[2 * ind + 1] != preInd):
                    break
                preInd = ind
                preBase = self.dat[2 * ind]
                ind = preBase
                if(not (ind < 0 or ind >= self.datSize or self.dat[2 * ind + 1] != preInd)):
                    bs.append(offset)
                    es.append(i + 1)
                    if(empty):
                        empty = False
        return not empty
    
    def match(self, word):
        ind = 0
        base = 0
        for i in range(len(word)):
            ind = self.dat[2 * ind] + ord(word[i])
            if((ind > self.datSize) or (self.dat[2 * ind + 1] != base)):
                return -1
            base = ind
        ind = self.dat[2 * base]
        if((ind < self.datSize) and (self.dat[2 * ind + 1] == base)):
            return ind
        return -1

    def update(self, word, value):
        base = self.match(word)
        if(base >= 0):
            self.dat[2 * base] = value

    def getInfo(self, prefix):
        ind = 0
        base = 0
        for i in range(len(prefix)):
            ind = self.dat[2 * ind] + ord(prefix[i])
            if((ind >= self.datSize) or self.dat[2 * ind + 1] != base):
                return i
            base = ind
        return -base

    def getDatSize(self):
        return self.datSize
    
    def getDat(self):
        return self.dat
    
if __name__ == '__main__':
    dat = Dat("cws_dat.bin")
    
        



# coding = utf-8
import os
import struct


class Preprocesser:
    def __init__(self):
        self.otherSet = []
        self.singlePunSet = []
        self.httpSet = []
        self.t2s = {}
        self.s2t = {}
        for i in range(65, 91):
            self.otherSet.append(i)
            self.httpSet.append(i)
        for i in range(97, 123):
            self.otherSet.append(i)
            self.httpSet.append(i)
        for i in range(48, 58):
            self.otherSet.append(i)
            self.httpSet.append(i)
        other = [65292, 12290, 65311, 65281, 65306, 65307, 8216, \
                8217, 8220, 8221, 12304, 12305, \
                12289, 12298, 12299, 126, 183, 64, 124, 35, 65509, 37, 8230, 38, 42, 65288, \
                65289, 8212, 45, 43, 61, 44, 46, 60, 62, 63, 47, 33, 59, 58, 39, 34, 123, 125, \
                91, 93, 92, 124, 35, 36, 37, 94, 38, 42, 40, 41, 95, 45, 43, 61, 9700, 9734, 9733]
        templen = 63
        for i in range(templen):
            self.otherSet.append(other[i])

        singlePun = [65292, 12290, 65311, 65281, 65306, 65307, 8216, 8217, 8220, 8221, 1230, 12304, \
                    12305, 12289, 12298, 12299, 64,35, 65288, 65289, 34, 91, 93, 126, 47, 44, 58, \
                    63, 9700, 9734, 9733, 8230, 39, 33, 42, 43, 62, 40, 41, 59, 61]
        templen = 41
        for i in range(templen):
            self.singlePunSet.append(singlePun[i])

        httpChar = ['/', '.', ':', '#', '"', '_', '-', '=', '+', '&', '$', ';']
        templen = 12
        for i in range(templen):
            self.httpSet.append(ord(httpChar[i]))

    def isOther(self, c):
        if(c in self.otherSet):
            return True
        else:
            return False
    
    def isSinglePun(self, c):
        if(c in self.singlePunSet):
            return True
        else:
            return False
    
    def isHttp(self, c):
        if(c in self.httpSet):
            return True
        else:
            return False

    def setT2SMap(self, filename):
        file = open(filename, "rb")
        self.datSize = int(os.path.getsize(filename) / 8)
        tempbytes = file.read(4 * self.datSize)
        tra = struct.unpack("<"+str(self.datSize)+"i", tempbytes)
        tempbytes = file.read(4 * self.datSize)
        sim = struct.unpack("<"+str(self.datSize)+"i", tempbytes)
        for i in range(self.datSize):
            self.t2s[tra[i]] = sim[i]
            self.s2t[sim[i]] = tra[i]

    def clean(self, sentence):
        senClean = ""
        graph = []
        hasSpace = False
        hasOther = False
        hasSinglePun = False
        hasHttp = False
        hasAt = False
        hasTitle = False
        httpStartVec = []
        httpStart = -1
        httpVec = []
        c = -1
        tmpRaw = []
        npRaw = []
        npStart = -1
        npStartVec = []
        npVec = []
        titleRaw = []
        titleStart = -1
        titleStartVec = []
        titleVec = []
        for i in range(len(sentence)):
            c = ord(sentence[i])
            if(c == 32 or c == 12288):
                hasOther = False
                if(hasSpace):
                    continue
                else:
                    if(len(graph) > 0):
                        o = graph[-1] & 12
                        graph[-1] = o
                    hasSpace = True
                    continue

                # if(hasAt):
                #     npVec.append(npRaw)
                #     npStartVec.append(npStart)
                #     hasAt = False
            elif(self.isOther(c)):
                if(hasSpace):
                    senClean += sentence[i]
                    if(self.isSinglePun(c)):
                        graph.append(8)
                        hasSinglePun = True
                    else:
                        graph.append(9)
                        hasSinglePun = False
                    hasSpace = False
                elif(hasOther):
                    if(self.isSinglePun(c)):
                        if(len(graph) > 0):
                            o = graph[-1] & 12
                            graph[-1] = o
                        senClean += sentence[i]
                        graph.append(8)
                        hasSinglePun = True

                    else:
                        if(hasSinglePun):
                            senClean += sentence[i]
                            graph.append(9)
                        else:              
                            if(graph[-1] == 0):
                                graph[-1] = 7
                            senClean += sentence[i]
                            graph.append(2)
                        hasSinglePun = False
                else:
                    senClean += sentence[i]
                    graph.append(9)
                    if(self.isSinglePun(c)):
                        hasSinglePun = True
                    else:
                        hasSinglePun = False

                # if(c == 41 or c == 65289):
                #     if(hasAt):
                #         npVec.append(npRaw)
                #         npStartVec.append(npStart)
                #         hasAt = False
                if(c == 12299):
                    if(hasTitle):
                        titleVec.append(titleRaw)
                        titleStartVec.append(titleStart)
                        hasTitle = False
                hasOther = True
            else:
                if(hasSpace):
                    senClean += sentence[i]
                    graph.append(9)
                elif(hasOther):
                    graph[-1] = graph[-1] & 12
                    if(hasSinglePun):
                        senClean += sentence[i]
                        graph.append(9)
                        hasSinglePun = False
                    else:  
                        senClean += sentence[i]
                        graph.append(15)
                else:
                    senClean += sentence[i]
                    graph.append(15)
                hasSpace = False
                hasOther = False

            # if(self.isHttp(c)):
            #     if(not hasHttp):
            #         if(c == 'h'):
            #             httpStart = len(graph) - 1
            #             tmpRaw = []
            #             tmpRaw.append(c)
            #             hasHttp = True
            #     else:
            #         tmpRaw.append(c)
            # else:
            #     if(hasHttp):
            #         httpVec.append(tmpRaw)
            #         httpStartVec.append(httpStart)
            #         hasHttp = False

            # if(c == 64):
            #     if(hasAt):
            #         npVec.append(npRaw)
            #         npStartVec.append(npStart)
            #         npRaw = []
            #     hasAt = True
            #     npStart = len(graph)
            #     npRaw = []
            # elif(hasAt):
            #     npRaw.append(c)

            if(c == 12298):
                hasTitle = True
                titleStart = len(graph)
                titleRaw = []
            elif(hasTitle):
                titleRaw.append(c)
        # if(len(tmpRaw) != 0):
        #     httpVec.append(tmpRaw)
        #     httpStartVec.append(httpStart)
        # if(len(npRaw) != 0):
        #     npVec.append(npRaw)
        #     npStartVec.append(npStart)

        # tempstr = ""
        # for i in range(len(httpVec)):
        #     tempstr = "".join(httpVec[i])
        #     found = tempstr.find("http")
        #     if(found != -1):
        #         start = httpStartVec[i]
        #         size = len(tempstr)
        #         # //std::cout<<std::endl<<sentence<<":Here:"<<str<<":"<<start<<":"<<size<<":"<<graph.size()<<std::endl;
        #         graph[start] = 1
        #         # for(int j = start + 1; j < start + size - 1; j ++){
        #         for j in range(start+1, start + size - 1):
        #             graph[j] = 2
        #         graph[start + size - 1] = 4

        # for i in range(len(npVec)):
        #     npRaw = npVec[i]
        #     if(len(npRaw) < 15):
        #         start = npStartVec[i]
        #         size = len(npRaw)

        #         graph[start] = 1
        #         # for(int j = start + 1; j < start + size - 1; j ++){
        #         for j in range(start + 1, start + size -1):
        #             graph[j] = 2
        #         graph[start + size - 1] = 4

        for i in range(len(titleVec)):
            titleRaw = titleVec[i]
            if(self.isPossibleTitle(titleRaw)):
                start = titleStartVec[i]
                size = len(titleRaw)
                # print sentence + ":Here" + str(titleRaw) + ":" + str(start) + ":" + str(size) + ":" + str(len(graph))
                graph[start] = 1
                for j in range(start + 1, start + size - 1):
                    graph[j] = 2
                graph[start + size - 1] = 4
        
        if(len(graph) != 0):
            graph[0] = graph[0] & 9
            graph[-1] = graph[-1] & 12
            if(graph[0] == 0):
                graph[0] = 9
            if(graph[-1] == 0):
                graph[-1] = 12
        return senClean, graph

    def isPossibleTitle(self, titleRaw):
        if(len(titleRaw) > 10 or len(titleRaw) == 0):
            return False
        else:
            for i in range(len(titleRaw)):
                if(self.isOther(titleRaw[i])):
                    return False
            return True

    def getT2S(self, c):
        if(c in self.t2s):
            return t2s[c]
        else:
            return c

    def getS2T(self, c):
        if(c in self.s2t):
            return s2t[c]
        else:
            return c

    def T2S(self, sentence):
        newSentence = ""
        for i in range(sentence):
            newSentence += str(getT2S(sentence[i]))
        return newSentence
    
    def S2T(self, sentence, oriSentence):
        count = 0
        for i in range(len(sentence)):
            for j in range(len(sentence[i].word)):
                sentence[i].word = sentence[i].word[0, j-1]+oriSentence[count] \
                        + sentence[i].word[j+1:]
                count = count + 1

    def cleanSpace(self, sentence, senClean, graph):
        senClean = ""
        graph = []
        hasSpace = False
        c = -1
        wordLength = 0
        # for(int i=0;i<(int)sentence.length();i++)
        for i in range(len(sentence)):
            c = sentence[i]
            if(c==32 or c==12288):
                if(hasSpace):
                    continue
                else:
                    if(len(graph) > 0):
                        if(wordLength == 1):   
                            graph[-1] = 8
                        else:
                            graph[-1] = 4
                    hasSpace = True
                wordLength = 0
            else:
                if(hasSpace):
                    senClean += sentence[i]
                    graph.append(1)
                    hasSpace = False
                else:
                    senClean += sentence[i]
                    if(len(graph) == 0):
                        graph.append(1)
                    else:
                        graph.append(2)

                wordLength = wordLength + 1
        if(len(graph) > 0):
            if(wordLength == 1):   
                graph[-1] = 8
            else:
                graph[-1] = 4
        return graph

if __name__ == '__main__':
    preprocesser = Preprocesser()
    preprocesser.setT2SMap("t2s.dat")
    f = open("cs.txt", "r")
    s = f.readline().decode('utf-8')
    a = []
    b, a = preprocesser.clean(s)
    print(a)
    print(b.encode("utf-8"))



from .base.Dat import Dat


class Filter:
    def __init__(self, xuWordFile, timeWordFile):
        self.xu_dat = Dat(xuWordFile)
        self.time_dat = Dat(timeWordFile)
        self.posSet = ["n","np","ns","ni","nz","v","a","id","t","uw"]
        self.arabicNumSet = []
        self.chineseNumSet = [12295,19968,20108,19977,22235,20116,20845,19971,20843,20061]
        for i in range(48, 58):
            self.arabicNumSet.append(i)
        for i in range(65296, 65306):
            self.arabicNumSet.append(i)

    def adjustSeg(self, sentence):
        if((self.xu_dat is None) or (self.time_dat is None)):
            return
        size = len(sentence)
        count = 0
        checkArabic = False
        checkChinese = False
        for i in range(size-1, -1, -1):
            word = sentence[i]
            if(self.xu_dat.match(word) != -1):
                sentence.remove(word)
                continue
            count = 0
            checkArabic = False
            checkChinese = False
            for j in range(len(word)):
                if(ord(word[j]) in self.arabicNumSet):
                    checkArabic = True
                    break
                if(ord(word[j]) in self.chineseNumSet):
                    count = count + 1
                    if(count == 2):
                        checkChinese = True
                        break
            if(checkArabic or checkChinese or (self.time_dat.match(word) != -1)):
                sentence.remove(word)

    def adjustTag(self, sentence):
        if(self.xu_dat is None or self.time_dat is None):
            return
        size = len(sentence)
        word = ""
        tag = ""
        count = 0
        checkArabic = False
        checkChinese = False

        for i in range(size-1, -1, -1):
            word = sentence[i][0]
            tag = sentence[i][2]
            if(tag in self.posSet):
                if(self.xu_dat.match(word) != -1):               
                    sentence.remove(sentence[i])
                    continue
                if(tag == "t"):  
                    count = 0
                    checkArabic = False
                    checkChinese = False
            
                    for j in range(len(word)):
                        if(ord(word[j]) in self.arabicNumSet):
                            checkArabic = True
                            break
                        if(ord(word[j]) in self.chineseNumSet):
                            count = count + 1
                            if(count == 2):
                                checkChinese = True
                                break
                    if(checkArabic or checkChinese or (self.time_dat.match(word) != -1)):
                        sentence.remove(sentence[i])
                        continue
            else:
                sentence.remove(sentence[i])
                continue



#coding = utf-8
from .CBModel import CBModel
from .CBNGramFeature import CBNGramFeature
from .base.Node import Node
from .base.Dat import Dat
from .base.WordWithTag import WordWithTag
from .base.AlphaBeta import AlphaBeta
import time
import array

class CBTaggingDecoder:
    def __init__(self):
        self.separator = '_'
        self.maxLength = 10000
        self.len = 0
        self.sequence = ""
        self.allowedLabelLists = []
        for i in range(self.maxLength):
            self.allowedLabelLists.append([])
        self.pocsToTags = None
        self.nGramFeature = None
        self.dat = None
        self.nodes = [Node() for i in range(self.maxLength)]
        self.labelTrans = None
        self.labelTransPre = None
        self.labelTransPost = None
        self.threshold = 0

        self.allowCom = [0 for i in range(self.maxLength)]
        self.tagSize = 0
        self.model = None
        self.alphas = None
        # self.betas = None

    def init(self, modelFile, datFile, labelFile):
        self.model = CBModel(modelFile)
        self.values = {}
        self.result = {}
        for i in range(self.maxLength):
            pre = (i - 1)
            self.nodes[i].predecessors = pre

            pre = (i + 1)
            self.nodes[i].successors = pre
        
        self.dat = Dat(datFile)
        self.nGramFeature = CBNGramFeature(self.dat, self.model)
        
        self.labelInfo = ["" for i in range(10000)]
        self.pocTags = []
        for i in range(16):
            self.pocTags.append([])
        labelin = open(labelFile, "r")
        line = ""
        ind = 0
        line = labelin.readline()
        while(len(line) > 0):
            self.labelInfo[ind] = line
            segInd = int(line[0]) - int('0')
            for j in range(16):
                if(((1 << segInd) & j) != 0):
                    self.pocTags[j].append(ind)
            ind = ind + 1
            line = labelin.readline()
        labelin.close()

        self.pocsToTags = [[] for i in range(16)]
        for j in range(1, 16, 1):
            self.pocsToTags[j] = [0 for i in range(len(self.pocTags[j]) + 1)]
            for k in range(len(self.pocTags[j])):
                self.pocsToTags[j][k] = self.pocTags[j][k]
            self.pocsToTags[j][len(self.pocTags[j])] = -1

        self.labelLookingFor = [[] for i in range(self.model.l_size)]
        for i in range(self.model.l_size):
            self.labelLookingFor[i] = None
        for i in range(self.model.l_size):
            if(self.labelInfo[i][0] == '0' or self.labelInfo[i][0] == '3'):
                continue
            for j in range(i + 1):
                if((self.labelInfo[i][1:] == self.labelInfo[j][1:]) and (self.labelInfo[j][0] == '0')):
                    if(self.labelLookingFor[j] is None):
                        self.labelLookingFor[j] = [0, 0]
                        self.labelLookingFor[j][0] = -1
                        self.labelLookingFor[j][1] = -1
                        self.tagSize = self.tagSize + 1
                    self.labelLookingFor[j][int(self.labelInfo[i][0])-1] = i
                    break

        for i in range(self.maxLength):
            self.allowedLabelLists[i] = None
        self.isGoodChoice = [0 for i in range(self.maxLength * self.model.l_size)]
        print("Model loaded succeed")

    def dp(self):
        if(self.allowedLabelLists[0] is None):
            self.allowedLabelLists[0] = self.pocsToTags[9]
        if(self.allowedLabelLists[self.len - 1] is None):
            self.allowedLabelLists[self.len - 1] = self.pocsToTags[12]
        alp = AlphaBeta()
        self.result = {}
        self.alphas = []
        self.bestScore = alp.dbDecode(self.model.l_size, self.model.ll_weights, self.len, self.nodes, self.values, self.alphas, self.result, self.labelTransPre, self.allowedLabelLists)
        self.allowedLabelLists[0] = None
        self.allowedLabelLists[self.len - 1] = None

    def setLabelTrans(self):
        lSize = self.model.l_size
        preLabels = [[] for i in range(lSize)]
        postLabels = [[] for i in range(lSize)]
        for i in range(lSize):
            for j in range(lSize):
                ni = int(self.labelInfo[i][0]) - 0
                nj = int(self.labelInfo[j][0]) - 0
                iIsEnd = ((ni == 2) or (ni == 3))
                jIsBegin = ((nj == 0) or (nj == 3))
                sameTag = self.labelInfo[i][1:] == self.labelInfo[j][1:]
                if(sameTag):
                    if((ni == 0 and nj == 1) or \
                            (ni == 0 and nj == 2) or \
                            (ni == 1 and nj == 2) or \
                            (ni == 1 and nj == 1) or \
                            (ni == 2 and nj == 0) or \
                            (ni == 2 and nj == 3) or \
                            (ni == 3 and nj == 3) or \
                            (ni == 3 and nj == 0)):
                        preLabels[j].append(i)
                        postLabels[i].append(j)
                else:
                    if(iIsEnd and jIsBegin):
                        preLabels[j].append(i)
                        postLabels[i].append(j)
        self.labelTransPre = [[] for i in range(lSize)]
        for i in range(lSize):
            self.labelTransPre[i] = [0 for k in range(len(preLabels[i])+1)]
            for j in range(len(preLabels[i])):
                self.labelTransPre[i][j] = preLabels[i][j]
            self.labelTransPre[i][len(preLabels[i])] = -1

        self.labelTransPost = [[] for i in range(lSize)]
        for i in range(lSize):
            self.labelTransPost[i] = [0 for k in range(len(postLabels[i])+1)]
            for j in range(len(postLabels[i])):
                self.labelTransPost[i][j] = postLabels[i][j]
            self.labelTransPost[i][len(postLabels[i])] = -1

    def putValues(self):
        if(self.len == 0):
            return
        for i in range(self.len):
            self.nodes[i].type = 0
        self.nodes[0].type += 1
        self.nodes[self.len-1].type += 2
        tmp = [0 for i in range(self.len * self.model.l_size)]
        self.values = array.array("i",  tmp)
        self.nGramFeature.putValues(self.sequence, self.len, self.values)
        self.values = tuple(self.values)

    def segmentTag(self, raw, graph):
        if(len(raw) == 0):
            return 0, []
        for i in range(len(raw)):
            pocs = graph[i]
            if(pocs != 0):
                self.allowedLabelLists[i] = self.pocsToTags[pocs]
            else:
                self.allowedLabelLists[i] = self.pocsToTags[15]
        self.sequence = raw
        self.len = len(raw)
        start = time.clock()
        self.putValues()
        self.dp()

        offset = 0
        if(len(self.labelInfo[0]) < 2):
            return 1, []
        ts = []
        # print self.result
        for i in range(self.len):
            if(i not in self.result):
                self.result[i] = 0
            if((i == self.len-1) or (self.labelInfo[self.result[i]][0] == '2') or (self.labelInfo[self.result[i]][0] == '3')):
                ts.append((self.sequence[offset:i+1], self.separator, self.labelInfo[self.result[i]][1:-1]))
                offset = i + 1
        return 1, ts

    def get_seg_result(self):
        segged = []
        offset = 0
        for i in range(self.len):
            if((i == 0) or (self.labelInfo[self.result[i]][0] == '0') or (self.labelInfo[self.result[i]][0] == '3')):
                segged.append(self.sequence[offset:i])
                offset = i
        segged.append(self.sequence[offset:self.len])
        return segged[1:]
